//+------------------------------------------------------------------+
//|                                                        dtosc.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"


#property indicator_separate_window
#property indicator_buffers 2
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_width1  2
#property indicator_level1  30
#property indicator_level2  70

//
//
//
//
//

   extern string TimeFrame      = "Current time frame";
   extern int    PeriodRSI      = 13;
   extern int    PeriodStoch    =  8;
   extern int    PeriodSK       =  5;
   extern int    PeriodSD       =  3;
   extern int    MAMode         =  0;
   extern double TMSmoothing    =  5;
   extern double TMDamping      =  5;
   extern double TMNoise        = 30;
   extern bool   arrowsVisible  = false;
   extern string arrowsUniqueID = "DtoscTMArrows1";
   extern color  arrowsUpColor  = DeepSkyBlue;
   extern color  arrowsDnColor  = PaleVioletRed;
   extern int    arrowsUpCode   = 241;
   extern int    arrowsDownCode = 242;
   extern bool   Interpolate    = true;
   
//
//
//
//
//

double SK[];
double SD[];
double StoRSI[];
double RSI[];
double trend[];
string IndicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
      SetIndexBuffer(0,SK);
      SetIndexBuffer(1,SD);
      SetIndexBuffer(2,StoRSI);
      SetIndexBuffer(3,RSI);
      SetIndexBuffer(4,trend);

   //
   //
   //
   //
   //
   
      IndicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);   

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" dtosc ("+PeriodRSI+","+PeriodStoch+","+PeriodSK+","+PeriodSD+")");
   return(0);
}

//
//
//
//
//

int deinit()
{
   string lookFor       = arrowsUniqueID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit;
   int counted_bars = IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
             limit=MathMin(Bars-counted_bars,Bars-1);
             if (returnBars) { SK[0] = limit; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--)
      {
         RSI[i] = iRSI(NULL,0,PeriodRSI,PRICE_CLOSE,i);
         double lo = RSI[ArrayMinimum(RSI,PeriodStoch,i)];
         double hi = RSI[ArrayMaximum(RSI,PeriodStoch,i)];
         if (hi!=lo)
               StoRSI[i] = MathMax(MathMin(iTmSmooth(100.0*(RSI[i] - lo)/(hi - lo),TMDamping,TMNoise,TMSmoothing,i),100),0);
         else  StoRSI[i] = MathMax(MathMin(iTmSmooth(0                            ,TMDamping,TMNoise,TMSmoothing,i),100),0);      
      }   
      for(i=limit; i>=0; i--) SK[i]=iMAOnArray(StoRSI,0,PeriodSK,0,MAMode,i);
      for(i=limit; i>=0; i--)
      {
         SD[i]=iMAOnArray(SK,0,PeriodSD,0,MAMode,i);
         trend[i] = trend[i+1];
            if (SK[i] > SD[i]) trend[i] =  1;
            if (SK[i] < SD[i]) trend[i] = -1;
            manageArrow(i);
      }         
      return(0);      
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         SK[i]    = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",PeriodRSI,PeriodStoch,PeriodSK,PeriodSD,MAMode,TMSmoothing,TMDamping,TMNoise,0,y);
         SD[i]    = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",PeriodRSI,PeriodStoch,PeriodSK,PeriodSD,MAMode,TMSmoothing,TMDamping,TMNoise,1,y);
         trend[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",PeriodRSI,PeriodStoch,PeriodSK,PeriodSD,MAMode,TMSmoothing,TMDamping,TMNoise,4,y);
            manageArrow(i);
            
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               SK[i+k] = SK[i] +(SK[i+n]-SK[i])*k/n;
               SD[i+k] = SD[i] +(SD[i+n]-SD[i])*k/n;
            }               
   }               

   //
   //
   //
   //
   //
             
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode  ,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDownCode,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsUniqueID+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsUniqueID+":"+time; ObjectDelete(lookFor);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double wrkTm[][3];
#define _price 0
#define _y1    1
#define _y2    2

//
//
//
//
//

double iTmSmooth(double price, double damping, double noise, double length, int i)
{
   if (ArrayRange(wrkTm,0) != Bars) ArrayResize(wrkTm,Bars);
   if (length<=1) return(price);
      int r = Bars-i-1;

   //
   //
   //
   //
   //

      wrkTm[r][_price] = price;

         double y1 = wrkTm[r-1][_y1];
         double y2 = wrkTm[r-1][_y2];
         
         double err = (wrkTm[r][_price]-2.0*y1+y2)/noise; 
         double drv = MathMax(MathMin((1.0/length)*err*err+(1.0/length)*MathAbs(err),0.5),0.0);
         double dmp = MathMax(MathMin((1.0/length)*MathAbs(y1-y2)/noise + damping/100.0,1.0),0.0);
         
         //
         //
         //
         //
         //
         
      double tmAvg  = y1 + noise*err*drv + (y1-y2)*(1.0-dmp);
      wrkTm[r][_y1] = tmAvg;
      wrkTm[r][_y2] = y1;
      return(tmAvg);
}   